==> eelbrain-0.38.1-cp39-cp39-macosx_10_9_x86_64.whl: eelbrain/_trf/_predictors.py <==
"""Predictors for deconvolution"""
# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
from itertools import repeat

import numpy as np

from .._data_obj import NDVar, Case, UTS, asndvar, asarray


def epoch_impulse_predictor(shape, value=1, latency=0, name=None, ds=None):
    """Time series with one impulse for each of ``n`` epochs

    Parameters
    ----------
    shape : NDVar | (int, UTS) | str
        Shape of the output. Can be specified as the :class:`NDVar` with the
        data to predict, or an ``(n_cases, time_dimension)`` tuple.
    value : scalar | sequence | str
        Scalar or length ``n`` sequence of scalars specifying the value of each
        impulse (default 1).
    latency : scalar | sequence | str
        Scalar or length ``n`` sequence of scalars specifying the latency of
        each impulse (default 0).
    name : str
        Name for the output :class:`NDVar`.
    ds : Dataset
        If specified, input items (``shape``, ``value`` and ``latency``) can be
        strings to be evaluated in ``ds``.

    See Also
    --------
    event_impulse_predictor : for continuous time series

    Examples
    --------
    See :ref:`exa-impulse` example.
    """
    if isinstance(shape, str):
        shape = asndvar(shape, ds=ds)
    if isinstance(value, str):
        value = asarray(value, ds=ds)
    if isinstance(latency, str):
        latency = asarray(latency, ds=ds)

    if isinstance(shape, NDVar):
        if not shape.has_case:
            raise ValueError(f'shape={shape!r}: has no case dimension')
        n = len(shape)
        time = shape.get_dim('time')
    else:
        n, time = shape
        if not isinstance(time, UTS):
            raise TypeError(f'shape={shape!r}: second item needs to be UTS instance')

    x = np.zeros((n, len(time)))
    t_index = time._array_index(latency)
    if np.isscalar(latency):
        x[:, t_index] = value
    else:
        x[np.arange(n), t_index] = value
    return NDVar(x, (Case, time), name)


def event_impulse_predictor(shape, time='time', value=1, latency=0, name=None, ds=None):
    """Time series with multiple impulses

    Parameters
    ----------
    shape : NDVar | UTS
        Shape of the output. Can be specified as the :class:`NDVar` with the
        data to predict, or an ``(n_cases, time_dimension)`` tuple.
    time : sequence of scalar
        Time points at which impulses occur.
    value : scalar | sequence
        Magnitude of each impulse (default 1).
    latency : scalar | sequence
        Latency of each impulse relative to ``time`` (default 0).
    name : str
        Name for the output :class:`NDVar`.
    ds : Dataset
        If specified, input items (``time``, ``value`` and ``latency``) can be
        strings to be evaluated in ``ds``.

    See Also
    --------
    event_impulse_predictor : for epoched data (with :class:`Case` dimension)
    """
    if isinstance(shape, NDVar):
        uts = shape.get_dim('time')
    elif isinstance(shape, UTS):
        uts = shape
    else:
        raise TypeError(f'shape={shape!r}')

    time, n = asarray(time, ds=ds, return_n=True)
    dt = uts.tstep / 2
    index = (time > uts.tmin - dt) & (time < uts.tstop - dt)
    if (sub_time := not index.all()):
        time = time[index]
    else:
        index = None

    if isinstance(value, str) or not np.isscalar(value):
        value = asarray(value, sub=index, ds=ds, n=n)
    else:
        value = repeat(value)

    if isinstance(latency, str) or not np.isscalar(latency):
        latency = asarray(latency, sub=index, ds=ds, n=n)
    else:
        latency = repeat(latency)

    out = NDVar(np.zeros(len(uts)), uts, name)
    for t, l, v in zip(time, latency, value):
        out[t + l] = v
    return out

==> eelbrain-0.39.4-cp39-cp39-macosx_10_9_x86_64.whl: eelbrain/_trf/_predictors.py <==
"""Predictors for continuous data"""
# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
from itertools import repeat
from typing import Sequence, Tuple, Union

import numpy as np

from .._data_obj import NDVarArg, NDVar, Case, Dataset, UTS, asndvar, asarray
from .._utils import deprecate_ds_arg


@deprecate_ds_arg
def epoch_impulse_predictor(
        shape: Union[NDVarArg, Tuple[int, UTS]],
        value: Union[float, Sequence[float], str] = 1,
        latency: Union[float, Sequence[float], str] = 0,
        name: str = None,
        data: Dataset = None,
) -> NDVar:
    """Time series with one impulse for each of ``n`` epochs

    Parameters
    ----------
    shape
        Shape of the output. Can be specified as the :class:`NDVar` with the
        data to predict, or an ``(n_cases, time_dimension)`` tuple.
    value
        Scalar or length ``n`` sequence of scalars specifying the value of each
        impulse (default 1).
    latency
        Scalar or length ``n`` sequence of scalars specifying the latency of
        each impulse (default 0).
    name
        Name for the output :class:`NDVar`.
    data
        If specified, input items (``shape``, ``value`` and ``latency``) can be
        strings to be evaluated in ``data``.

    See Also
    --------
    event_impulse_predictor : for continuous time series

    Examples
    --------
    See :ref:`exa-impulse` example.
    """
    if isinstance(shape, str):
        shape = asndvar(shape, data=data)
    if isinstance(value, str):
        value = asarray(value, data=data)
    if isinstance(latency, str):
        latency = asarray(latency, data=data)

    if isinstance(shape, NDVar):
        if not shape.has_case:
            raise ValueError(f'shape={shape!r}: has no case dimension')
        n = len(shape)
        time = shape.get_dim('time')
    else:
        n, time = shape
        if not isinstance(time, UTS):
            raise TypeError(f'shape={shape!r}: second item needs to be UTS instance')

    x = np.zeros((n, len(time)))
    t_index = time._array_index(latency)
    if np.isscalar(latency):
        x[:, t_index] = value
    else:
        x[np.arange(n), t_index] = value
    return NDVar(x, (Case, time), name)


@deprecate_ds_arg
def event_impulse_predictor(
        shape: Union[NDVarArg, UTS],
        time: Union[str, Sequence[float]] = 'time',
        value: Union[float, Sequence[float], str] = 1,
        latency: Union[float, Sequence[float], str] = 0,
        name: str = None,
        data: Dataset = None,
) -> NDVar:
    """Time series with multiple impulses

    Parameters
    ----------
    shape
        Shape of the output. Can be specified as the :class:`NDVar` with the
        data to predict, or an ``(n_cases, time_dimension)`` tuple.
    time
        Time points at which impulses occur.
    value
        Magnitude of each impulse (default 1).
    latency
        Latency of each impulse relative to ``time`` (default 0).
    name
        Name for the output :class:`NDVar`.
    data
        If specified, input items (``time``, ``value`` and ``latency``) can be
        strings to be evaluated in ``data``.

    See Also
    --------
    epoch_impulse_predictor : for epoched data (with :class:`Case` dimension and a single impulse per epoch)
    """
    if isinstance(shape, NDVar):
        uts = shape.get_dim('time')
    elif isinstance(shape, UTS):
        uts = shape
    else:
        raise TypeError(f'shape={shape!r}')

    time, n = asarray(time, data=data, return_n=True)
    dt = uts.tstep / 2
    index = (time > uts.tmin - dt) & (time < uts.tstop - dt)
    if index.all():
        index = None
    else:
        time = time[index]

    if isinstance(value, str) or not np.isscalar(value):
        value = asarray(value, sub=index, data=data, n=n)
    else:
        value = repeat(value)

    if isinstance(latency, str) or not np.isscalar(latency):
        latency = asarray(latency, sub=index, data=data, n=n)
    else:
        latency = repeat(latency)

    out = NDVar(np.zeros(len(uts)), uts, name)
    for t, l, v in zip(time, latency, value):
        t_ = t + l
        if uts.tmin <= t_ <= uts.tmax:
            out[t_] = v
    return out
